package oop2_dat2_u;

/**
 *
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 */
public class MyLinkedList<Object>
{

    private MyListNode<Object> head; // Kopf bzw. Anfang der Liste
    private MyListNode<Object> tail; // Schwanz bzw. Ende der Liste
    private int numOfElements; // Anzahl Elemente in dieser Liste

    public MyLinkedList()
    {
        head = null;
        tail = null;
        numOfElements = 0;
    }

    /**
     * Prüft, ob Liste leer ist.
     */
    public boolean isEmpty()
    {
        return (head == null);
    }

    /**
     * Fügt Objekt obj am Anfang in die Liste ein.
     */
    public void insert(Object obj)
    {
        // aktuelles Element zwischenspeichern
        MyListNode<Object> actualNode = head;

        // neues Objekt einfügen
        head = new MyListNode<Object>(head, obj);

        // Liste wieder zusammenhängen, head / tail updaten
        if (actualNode != null) {
            actualNode.setPrev(head);
        } else {
            tail = head;
        }
        numOfElements ++;
    }

    /**
     * Prüft, ob ein gleiches Objekt wie obj bereits in der Liste enthalten ist.
     */
    public boolean isFound(Object obj)
    {
        MyListNode<Object> actualNode = head;
        while ((actualNode != null) &&  ! obj.equals(actualNode.getData())) {
            actualNode = actualNode.getNext();
        }
        if (actualNode == null) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Entfernt aus der Liste das erste Objekt gleich obj.
     */
    public void remove(Object obj)
    {
        MyListNode<Object> actualNode = head;
        MyListNode<Object> prevNode = null;
        while ((actualNode != null) &&  ! obj.equals(actualNode.getData())) {
            prevNode = actualNode;
            actualNode = prevNode.getNext();
        }
        // Liste wieder ordentlich zusammenhängen:
        if (actualNode != null) {
            if (actualNode == head) {
                head = actualNode.getNext();
                head.setPrev(null);
            } else {
                prevNode.setNext(actualNode.getNext());
                actualNode.getNext().setPrev(prevNode);
            }
            numOfElements --;
        }
    }

    /**
     * Gibt alle Objekte der Reihe nach zeilenweise auf die Konsole aus.
     */
    public void print()
    {
        MyListNode<Object> actualNode = head;
        while (actualNode != null) {
            System.out.println(actualNode.getData());
            actualNode = actualNode.getNext();
        }
    }

    /**
     * Gibt alle Objekte rückwärts der Reihe nach zeilenweise auf die Konsole aus.
     */
    public void printReverse()
    {
        MyListNode<Object> actualNode = tail;
        while (actualNode != null) {
            System.out.println(actualNode.getData());
            actualNode = actualNode.getPrev();
        }
    }

    public int size()
    {
        return numOfElements;
    }
}
